import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Metadata, Http, I18n } from '@zeedhi/core';
import {
  ModalService, TextInput, Form, LoadingService,
} from '@zeedhi/common';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';

export class PriceAdjustmentController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/priceAdjustmentModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('priceAdjustmentModal')?.destroy();
  }

  public openModal(): boolean {
    const quotationGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');

    if (!quotationGrid.datasource.currentRow.NRCOLEPRECO
        && !regionGrid.datasource.currentRow.CDREGIAO
        && !supplierGrid.datasource.currentRow.CDFORNECED
    ) {
      this.utilities.showDialog(
        'Selecione um fornecedor para ajustar preço dos produtos.',
        'warning',
      );

      return false;
    }

    ModalService.getModal('priceAdjustmentModal').show();

    setTimeout(() => {
      const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      const quotation = Metadata.getInstance<TextInput>(
        'priceAdjustmentModalQuotation',
      );

      quotation.value = grid.datasource.currentRow.NRCOLEPRECO;
    }, 100);

    return true;
  }

  public closeModal(): void {
    ModalService.getModal('priceAdjustmentModal').hide();
  }

  public async savePriceAdjustment(): Promise<void> {
    const form = Metadata.getInstance<Form>('priceAdjustmentModalForm');
    const isValid = form.validate();

    if (!isValid) return;

    LoadingService.show();

    try {
      const parameters = this.prepareParams(form);
      const { data } = await Http.put('/adjustPrice', { params: parameters });

      if (data.error) {
        this.utilities.showDialog(data.error, 'error');

        return;
      }

      LoadingService.hide();
      this.closeModal();
      this.utilities.showDialog('sup::SUCCESS', 'success');

      const grid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
      grid.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  private prepareParams(form: Form) {
    const quotationGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');

    return {
      CDFILCENT: this.appState.CDFILCENTCMP,
      CDFILIAL: quotationGrid.datasource.currentRow.CDFILIAL,
      NRCOLEPRECO: quotationGrid.datasource.currentRow.NRCOLEPRECO,
      CDFORNECED: supplierGrid.datasource.currentRow.CDFORNECED,
      CDREGIAO: regionGrid.datasource.currentRow.CDREGIAO,
      DTCOLEPREC: quotationGrid.datasource.currentRow.DTCOLEPREC,
      CDOPERINCL: this.appState.CDOPERADOR,
      CDPRODINI: form.value.priceAdjustmentInitProduct
        ? form.value.priceAdjustmentInitProduct
        : null,
      CDPRODFIN: form.value.priceAdjustmentEndProduct
        ? this.utilities.addCaracterInRight(
          form.value.priceAdjustmentEndProduct,
        )
        : null,
      TODOSPROD:
        form.value.priceAdjustmentInitProduct
        && form.value.priceAdjustmentInitProduct
          ? null
          : 'T',
      REAJUSTE: form.value.priceAdjustment,
      DESCRICAO: I18n.translate('Botão Reajustar Preços - de / para: '),
      NRORG: this.appState.NRORG,
    };
  }
}
