import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService,
  LoadingService,
  TextInput,
  Select,
  Form,
} from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';
import type { ICreateRemoteReport } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class QuotationLogController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/quotationLogModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('quotationLogModal')?.destroy();
  }

  public openModal(): void {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!quoteGrid.selectedRows.length || quoteGrid.selectedRows.length > 1) {
      this.utilities.showDialog(
        'Selecione apenas uma cotação para acessar Log de Cotação',
        'warning',
      );
      return;
    }

    ModalService.getModal('quotationLogModal').show();

    setTimeout(async () => {
      await this.setFields();
    }, 200);
  }

  public closeModal(): void {
    ModalService.getModal('quotationLogModal').hide();
  }

  private async setFields(): Promise<void> {
    const quoteInput = Metadata.getInstance<TextInput>('quotation_log_quotation');
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    quoteInput.value = quoteGrid.selectedRows[0].NRCOLEPRECO;

    LoadingService.show();

    try {
      const reports = await this.utilities.getReports('SUP070');
      const birtReports = reports.filter((report) => report.strategy === 'BIRT');
      const reportField = Metadata.getInstance<Select>('quotation_log_report');

      reportField.datasource.data = birtReports;
      reportField.setValue('SUP07000.BIRT');
    } catch (error) {
      console.error(error);
      this.utilities.showDialog(
        'Erro ao pegar relatórios.',
        'error',
      );
    } finally {
      LoadingService.hide();
    }
  }

  public async generateReport(): Promise<void> {
    const form = Metadata.getInstance<Form>('quotation_log_form');

    if (!form.validate()) return;

    const parameters = this.prepareParams(form);

    if (form.value.quotation_log_report_type === 'XLSX') {
      const fileName = form.value.quotation_log_report.value.replace('.', '_');

      this.utilities.downloadReportXLSX(fileName, parameters);
    } else {
      this.utilities.openReport(parameters);
    }
  }

  private prepareParams(form: Form): ICreateRemoteReport {
    return {
      DTINI: form.value.quotation_log_date[0] ?? '13/02/1997',
      DTFIM: form.value.quotation_log_date[1] ?? '13/02/1997',
      P_DATAINIC: form.value.quotation_log_date[0] ?? '',
      P_DATAFINA: form.value.quotation_log_date[1] ?? '',
      NR_COTACAO: form.value.quotation_log_quotation ?? null,
      OPERADOR: form.value.quotation_log_operators.length ? form.value.quotation_log_operators.join('_') : 'T',
      TYPE: form.value.quotation_log_report_type.toLowerCase(),
      NRORG: this.appState.NRORG,
      REPORT_NAME: form.value.quotation_log_report.value,
      __report_name: form.value.quotation_log_report.value.substring(0, 8),
      STRATEGY: form.value.quotation_log_report.strategy,
      isUrl: form.value.quotation_log_report_type === 'PDF',
    };
  }
}
