import { TekGrid } from '@zeedhi/teknisa-components-common';
import { LoadingService } from '@zeedhi/common';
import { Metadata, Http, IEventParam } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';

export class ValidateOperatorController {
  private appState: AppState;

  private utilities!: Utilities;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async validateOperator({ component }: IEventParam<TekGrid>): Promise<void> {
    LoadingService.show();

    try {
      const parameters = {
        ROWS: component.selectedRows.length
          ? [...component.selectedRows]
          : [component.datasource.currentRow],
        IDUTINDICECOL: this.appState.registeredParams.IDUTINDICECOL,
        CDCOMPRADOR: this.appState.CDOPERADOR,
      };
      const { data } = await Http.get('/validateOperator', { params: parameters });

      if (data.error) {
        this.utilities.showDialog(data.error, 'warning');

        if (component.editing) component.cancelEditedRows();
      }

      this.changeQuoteGridState(!data.error === false);
      this.changeProductsState(!data.error);
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    } finally {
      LoadingService.hide();
    }
  }

  private changeQuoteGridState(isEnable: boolean): void {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const disabledActions = [
      'updatePaymentMethod',
      'duplicateQuotation',
      'changeValidityQuotation',
      'loadPurchaseQuantity',
      'loadBestPrices',
      'quoteLog',
    ];

    disabledActions.forEach((action) => {
      const index = quoteGrid.actions.map((e) => e.name as any).indexOf(action);
      quoteGrid.actions[index].disabled = isEnable;
    });

    regionGrid.addButton = !isEnable;
    supplierGrid.addButton = !isEnable;
    regionGrid.deleteButton = isEnable ? 'none' : 'selection';
    supplierGrid.deleteButton = isEnable ? 'none' : 'selection';
  }

  private changeProductsState(isEnable: boolean): void {
    const productGrid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
    productGrid.selectable = isEnable;

    productGrid.actions.forEach((action) => {
      if (action.name !== 'excludeItemsWithZeroPrice') action.disabled = !isEnable;
    });

    productGrid.columns.forEach((column) => {
      if (column.name === 'VRPRECUNIITM' || column.name === 'DSJUSTITM') {
        column.editable = isEnable;
      }
    });

    productGrid.datasource.data = [];
  }

  public enableActions(): void {
    this.changeQuoteGridState(false);
    this.changeProductsState(false);
  }
}
